--=========================================================
--
-- フェードエフェクト：
-- 黒丸の列が下からせり上がってくる感じのやつ
--
--=========================================================
Fade_circles = {}
Fade_circles.new = function ()
	
	local this = {}
	
	this.fadeRate = 0.0
	this.phase = ""
	
	-- 描画時の位置調整用にウィンドウのサイズを確保しておく。
	this.windowWidth, this.windowHeight, this.windowFlags = love.window.getMode( )
	
	--=========================================================
	-- 開始
	--=========================================================
	this.start = function(self)

		self.fadeRate = 0.0
		self.phase = "out"
		
	end
	
	
	--=========================================================
	-- 更新
	--=========================================================
	this.update = function(self)
		
		local pace = 0.03
		
		-- フェードアウト
		if ( self.phase == "out" ) then
			
			self.fadeRate = self.fadeRate + pace
			
			if ( self.fadeRate >= 1.0 ) then
				self.fadeRate = 1.0
				self.phase = "in"
				
				self:changeScene()
			end
			
		-- フェードイン
		elseif ( self.phase == "in" ) then
			
			self.fadeRate = self.fadeRate - pace
			
			if ( self.fadeRate <= 0.0 ) then
				self.fadeRate = 0.0
				self.phase = ""
			end
			
		end
		
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		-- 呼び出された時点の描画色を保存しておく（呼び出し元に影響がないように後で戻す）
		local r,g,b,a = love.graphics.getColor()
		
		love.graphics.setColor (0,0,0,1) -- 黒丸なので黒。変更すると色が変わる。
		
		local distance = 30 -- 黒丸同士の距離
		for y = 0, self.windowHeight + distance, distance
		do
			for x = 0, self.windowWidth + distance, distance
			do
				
				local yRate = y / (self.windowHeight + distance)
				
				local radius = 0
				
				-- フェードの進行度に応じて黒丸の半径を決める。
				-- 下の方ほど早期に最大サイズになる。
				radius = distance/1.3 * (self.fadeRate * ( 1 + yRate ))
				
				if ( radius > distance ) then radius = distance end
				if ( radius < 0        ) then radius = 0 end
				
				-- 描画
				love.graphics.circle ( "fill", x, y, radius )
			end
		end
		
		-- 描画色を戻す
		love.graphics.setColor(r,g,b,a)
		
	end
	
	
	
	return this
end









