Fade_grids = {}
Fade_grids.new = function ()
    
    local this = {}

    -- フェードの進行度（0.0：フェードなし 1.0：画面真っ暗）
    this.fadeRate = 0.0
    
    -- 処理フェイズ：フェード「in」か「out」か。
    this.phase = ""
    
    
    -- ウィンドウのサイズを確認して、
    local windowWidth, windowHeight = love.window.getMode()

    -- 特定のサイズのグリッドで画面を敷き詰めていく
    this.gridSize = 25
    this.grids = {}
    
    -- 画面全体を敷き詰めるようにグリッドを作成
    for y = 0, windowHeight, this.gridSize do
        for x = 0, windowWidth, this.gridSize do

            -- グリッドの実体は「座標」
            this.grids [ #this.grids + 1 ] = {
                x = x, y = y
            }
            
        end
    end



    --=========================================================
    -- 開始
    --=========================================================
    this.start = function(self)
        
        -- 乱数初期化
        math.randomseed(os.time())
        
        self.fadeRate = 0.0
        self.phase = "out"

        -- グリッドをシャッフル：ランダムに埋まっていく感じになる。
        for i = #self.grids, 1, -1 do
            
            local j = math.random(1,i)
            
            local temp = self.grids[i]
            self.grids[i] = self.grids[j]
            self.grids[j] = temp

        end
    end


    --=========================================================
    -- 更新
    --=========================================================
    this.update = function(self)
        
        local pace = 0.03
        
        -- フェードアウト
        if ( self.phase == "out" ) then
            
            -- フェードの度合いを「進める」
            self.fadeRate = self.fadeRate + pace
            
            -- フェードアウト終了判定
            if ( self.fadeRate >= 1.0 ) then
                
                -- ハミ出しを一応抑えて
                self.fadeRate = 1.0
                
                -- 処理フェイズを「イン」に移行
                self.phase = "in"
                
                
                -- シーン遷移を実行
                -- 中味はシーンマネージャーから事前に受け取る
                self:changeScene()
                
                
            end
            
        -- フェードイン
        elseif ( self.phase == "in" ) then
            
            -- フェードの度合いを「戻す」
            self.fadeRate = self.fadeRate - pace
            
            -- フェードイン終了判定
            if ( self.fadeRate <= 0.0 ) then
                self.fadeRate = 0.0
                self.phase = ""
            end
            
        end
    end



    --=========================================================
    -- 描画
    --=========================================================
    this.draw = function(self)
        
        -- 現時点の描画色を保持しておく（後で戻す）
        local r,g,b,a = love.graphics.getColor()
        
        -- 描画色を「黒塗り」にする
        love.graphics.setColor(0,0,0,1)
        
        -- フェード進行度合い（全グリッド中、いくつまで黒塗りにするか？）
        local progress = #self.grids * self.fadeRate
        
        -- 全グリッドでループして、
        for i = 1, #self.grids, 1 do
            
            -- 現在のフェード進行度に応じた個数まで、黒塗りする
            if ( i <= progress ) then
                
                local x = self.grids[i].x
                local y = self.grids[i].y
                
                love.graphics.rectangle ( "fill", x, y, self.gridSize, self.gridSize )
                
            end
            
        end
        
        -- 描画色を戻しておく
        love.graphics.setColor(r,g,b,a)
        
    end
    
    return this
end
