--=========================================================
--
-- フェードエフェクト：
-- モザイク状になるやつ
--
--=========================================================
Fade_pixel = {}
Fade_pixel.new = function ()
	
	local this = {}
	
	this.fadeRate = 0.0
	
	this.windowWidth, this.windowHeight, this.windowFlags = love.window.getMode( )
	
	this.gridSize = 25
	this.grids = {}
	
	this.phase = ""
	
	-- シェーダーでピクセル化する。ピクセルのサイズを変数として受け取る。フェードの進行度に応じて変える。
	this.shaderPixel = love.graphics.newShader[[
		
		extern number fadeThickness;
		
		number pixel_amount = fadeThickness * 10;
		
		vec4 effect(vec4 color, Image tex, vec2 p, vec2 pc) {
			vec4 finTex = Texel(tex, p);
			vec2 np;
			float a, b, c;
			
			vec2 p1 = floor(p*(100/pixel_amount))/(100/pixel_amount);
			vec2 p2 = floor(p*(100/-pixel_amount))/(100/-pixel_amount);
			vec4 finTex2 = Texel(tex, p1);
			vec4 finTex3 = Texel(tex, p2);

			vec4 finTex4 = mix(finTex2, finTex3, 0.5);
			
			return finTex4;
		}
	
	]]
	
	this.shader = nil
	
	--=========================================================
	-- 開始
	--=========================================================
	this.start = function(self)

		self.fadeRate = 0.0
		self.phase = "out"
		
		this.shader = this.shaderPixel
		
	end
	
	--=========================================================
	-- 更新
	--=========================================================
	--this.update = function(self, in_out)
	this.update = function(self)
		
		local pace = 0.03

		-- フェードアウト
		if ( self.phase == "out" ) then
			
			self.fadeRate = self.fadeRate + pace
			
			if ( self.fadeRate >= 1.0 ) then
				self.fadeRate = 1.0
				self.phase = "in"
				
				self:changeScene()
			end
			
		-- フェードイン
		elseif ( self.phase == "in" ) then
			
			self.fadeRate = self.fadeRate - pace
			
			if ( self.fadeRate <= 0.0 ) then
				self.fadeRate = 0.0
				self.phase = ""
			end
			
		end
		
		-- フェードの進行度をシェーダーに渡す。ピクセルのサイズが変わる。
		self.shader:send("fadeThickness", self.fadeRate)
		
	end
	
	--=========================================================
	-- 描画
	--=========================================================
	this.draw = function(self)
		
		-- フェードの進行度に応じてシェーダーをセット。もしくは解除。
		if ( self.fadeRate > 0.01 ) then
			love.graphics.setShader(self.shader)
		else
			love.graphics.setShader()
		end
		
		
	end
	
	
	
	return this
end









